import { useSelector } from "react-redux";
import { DefaultButton, TitleSeparator, WarningButton } from "../../components";
import { UserRootState } from "../../store/type";
import { DEFAULT_AVATAR_IMG } from "../../constants/default";
import { ChangeEvent, TargetedEvent, useRef } from "preact/compat";
import { useState } from "react";
import { enumKeys, useAutosizeTextArea } from "../../utils";
import { SocialMediaEnum } from "../../types/common";
import { SocialMedia, UserInfo } from "../../../src/domains/User";
import './styles.css'
import { deleteUserAvatarService, patchUserAvatarService, patchUserInfoService } from "../../../src/services/users";
import { AxiosError } from "axios";
import { useDispatch } from "react-redux";
import { authAdded } from "../../features/auth/authSlice/authSlice";

interface PageState {
  avatar_file: File | null,
  avatar: string,
  upload_file_err_msg: string,
  file_input_key: string
}

function UserSettings() {
  const user = useSelector((state: UserRootState) => state.auth)
  const dispatch = useDispatch()

  const [pageState, setPageState] = useState<PageState>({
    avatar_file: null,
    avatar: user.avatar_picture,
    upload_file_err_msg: '',
    file_input_key: Math.random().toString(24)
  })

  const [isLoading, setIsLoading] = useState(false)

  const [form, setForm] = useState<UserInfo>({
    about: user.about,
    website: user.website,
    social_media: user.social_media
  })


  const textAreaRef = useRef<HTMLTextAreaElement>(null);
  useAutosizeTextArea(textAreaRef.current, form.about);

  function onChangeFormInput(e: ChangeEvent) {
    let event = e.target as HTMLInputElement;
    setForm({ ...form, [event.name]: event.value })
  }

  function handleTextAreaChange(e: ChangeEvent<HTMLTextAreaElement>): void {
    const val = e.target as HTMLTextAreaElement;

    setForm({ ...form, about: val.value })
  }

  function onAddSocialMediaClicked() {
    let tempArr: Array<SocialMedia> = form.social_media
    tempArr.push({ type: SocialMediaEnum.Facebook, value: '' })
    setForm({ ...form, social_media: tempArr })
  }

  function onChangeSocialMediaInput(e: ChangeEvent, idx: number) {
    let event = e.target as HTMLInputElement;
    let socmed = form.social_media;
    socmed[idx].value = event.value;
    setForm({ ...form, social_media: socmed })

  }

  function onChangeSocialMediaType(e: ChangeEvent, idx: number) {
    let event = e.target as HTMLOptionElement
    let socmed = form.social_media;
    socmed[idx].type = event.value;
    setForm({ ...form, social_media: socmed })
  }

  function onDeleteSelectedSocial(idx: number) {
    setForm({ ...form, social_media: form.social_media.filter((_, index) => index !== idx) })
  }

  async function onUploadAvatar(e: TargetedEvent) {
    setIsLoading(true)
    e.preventDefault();
    try {
      const form = new FormData();
      form.append('file', pageState.avatar_file!)
      const res = await patchUserAvatarService(form)
      let userStore = {
        ...user,
        avatar_picture: res.data.image_url
      }
      dispatch(authAdded(userStore));
      setIsLoading(false)
    } catch(err) {
      let error = err as AxiosError;
      console.log(error)
      setIsLoading(false)
    }

    setPageState({ ...pageState, file_input_key: Math.random().toString(24)})
    
  }

  async function onRemoveAvatar(e: TargetedEvent) {
    setIsLoading(true)
    e.preventDefault();
    try {
      await deleteUserAvatarService()
      let userStore = {
        ...user,
        avatar_picture: ''
      }
      dispatch(authAdded(userStore))
      setPageState({ ...pageState, avatar: ''})
      setIsLoading(false)
    } catch(err) {
      setIsLoading(false)
      const error = err as AxiosError
      console.log(error)
    }
  }

  async function onUpdateUserInfo(e: TargetedEvent) {
    setIsLoading(true)
    e.preventDefault();
    try {
      console.log(form)
      const res = await patchUserInfoService(form)
      let userData = {
        ...user,
        about: res.data.about,
        website: res.data.website,
        social_media: res.data.social_media
      }

      dispatch(authAdded(userData))
      setIsLoading(false)
    }catch(err) {
      setIsLoading(false)
      let error = err as AxiosError;
      alert(error)
    }
  }

  function onChangeUploadImage(e: ChangeEvent) {
    let event = e.target as HTMLInputElement;
    const file = event.files![0] as File;
    
    // if cancel button pressed
    if (!file) {
      return
    }
    
    if (file.type !== "image/png" && file.type !== "image/jpg" && file.type !== "image/jpeg" ) {
      setPageState({ ...pageState, upload_file_err_msg: 'ONLY ACCEPT JPEG/PNG/JPG FILE!!!!' })
      return
    }
    
    if (file.size > 10000000) {
      setPageState({ ...pageState, upload_file_err_msg: 'file size too large, max file size 10mb' })
      return
    }

    setPageState({ ...pageState, upload_file_err_msg: '', avatar_file: file, avatar: URL.createObjectURL(file)})
  }

  return (
    <div className={'main-content content'}>
      <div className={'mb-4'}>
        <h1 className={'text-2xl'}>User settings</h1>
        <TitleSeparator titleClasses="mt-8 " pageName={'avatar'} titleStyles={{ letterSpacing: .9 }} />
        <div style={{ padding: '0 15px 20px 0', float: 'left', maxWidth: '50%', width: 200 }}>
          <img
            src={pageState.avatar !== '' ? pageState.avatar : DEFAULT_AVATAR_IMG}
            style={{ aspectRatio: '1/1', objectFit: 'cover'}}
          />
        </div>
        <div className={'text-sm mb-5'} style={{ overflow: 'hidden' }}>
          <span className={'mr-3'}>Change avatar</span>
          <input name="uploadImage" key={pageState.file_input_key} type={"file"} accept={'image/*'} onChange={onChangeUploadImage} />
          {pageState.upload_file_err_msg &&
            <p className={'text-xs text-error mt-2'}>{pageState.upload_file_err_msg}</p>
          }
          <div className={'mt-2 text-tertiary'}>
            <p>File extension must be .jpeg, .png, .jpg</p>
            <p>File size max 10mb</p>
          </div>
        </div>
        <div className={'text-xs button-avatar-container'}>
          <DefaultButton 
            label="update" 
            className={isLoading ? 'pointer-events-none' : ''}
            style={{ paddingLeft: 36, paddingRight: 36, textTransform: 'uppercase'}} 
            isLoading={isLoading}
            onClick={onUploadAvatar}
          />
          <WarningButton 
            label="remove avatar" 
            containerClassName="mt-5" 
            style={{ textTransform: 'uppercase', paddingLeft: 8, paddingRight: 8 }} 
            className={isLoading ? 'pointer-events-none' : ''}
            onClick={onRemoveAvatar}
          />
        </div>
        <div style={{ clear: 'both' }} />
      </div>

      <section name="User Info">
        <TitleSeparator pageName="INFO" />
        <div className={'mb-3'} style={{ maxWidth: 600, minWidth: 280 }}>
          <h2 className={'text-sm mb-1 font-bold'}>ABOUT YOU</h2>
          <textarea
            onChange={handleTextAreaChange}
            maxLength={500}
            ref={textAreaRef}
            className={'p-2 text-sm bg-secondary'}
            value={form.about}
            style={{ border: 'none', overflow: 'auto', outline: 'none', boxShadow: 'none', width: '100%', minHeight: 100, overflowY: 'hidden' }}
          />
          <div className={'mt-2 mb-1'}>
            <h2 className={'font-bold text-sm mb-1'}>WEBSITE</h2>
            <input className={'bg-secondary text-sm'} placeholder={'www.mywebsite.com'} style={{ width: '100%', borderRadius: 7, padding: '5px 10px' }} type={'text'} onChange={onChangeFormInput} name={'website'} value={form.website} />
          </div>
          <h2>Social</h2>
          {form.social_media.map((val, index) => (
            <div className={'mt-2'}>
              <select
                onChange={(e) => onChangeSocialMediaType(e, index)}
                className={'bg-secondary inline-block text-sm'}
                name="social_media_type"
                id="social_media_type"
                style={{ borderRadius: 7, padding: '5px 10px' }}
                value={form.social_media[index].type.toLowerCase()}
              >
                {enumKeys(SocialMediaEnum).map(x => (
                  <option value={SocialMediaEnum[x]}>{SocialMediaEnum[x]}</option>
                ))
                }
              </select>
              <input
                className={'bg-secondary text-sm ml-1 mr-1 inline-block'}
                style={{ width: '66%', borderRadius: 7, padding: '5px 10px' }}
                type={'text'}
                onChange={(e) => onChangeSocialMediaInput(e, index)}
                name={'social_media'}
                value={val.value} />
              <span className={'text-xs text-gray mr-3'}>( username )</span>
              <span
                className={'text-sm bg-error'}
                style={{ padding: '0 5px' }}
              >
                <a onClick={() => onDeleteSelectedSocial(index)} style={{ verticalAlign: 'text-bottom' }}>x</a>
              </span>
            </div>
          ))
          }
          <a onClick={onAddSocialMediaClicked}>
            <div className={'text-center mt-2 bg-secondary'} style={{ padding: '5px 10px', borderRadius: 7 }}>
              + add social media
            </div>
          </a>
          <div className={'flex flex-row justify-between mt-7'}>
            <DefaultButton 
              label="update" 
              containerClassName="text-xs" 
              onClick={onUpdateUserInfo}
              style={{ paddingLeft: 36, paddingRight: 36, textTransform: 'uppercase', }} 
              className={isLoading ? 'pointer-events-none' : ''}
              isLoading={isLoading}
            />
            <WarningButton 
              label="delete account" 
              containerClassName="text-xs" 
              style={{ textTransform: 'uppercase', paddingLeft: 8, paddingRight: 8 }} 
              isLoading={isLoading}
              className={isLoading ? 'pointer-events-none' : ''}
            />
          </div>
        </div>
      </section>
    </div>
  )
}

export default UserSettings;